#include <Keypad.h>
#include <LiquidCrystal.h>

const byte ROWS = 4; //4 rækker
const byte COLS = 3; //3 kolonner
// Symboler på Keypad
char hexaKeys[ROWS][COLS] = {
  {'1','2','3'},
  {'4','5','6'},
  {'7','8','9'},
  {'*','0','#'}
};
byte rowPins[ROWS] = {12, 11, 10, 9}; //connect to the row pinouts of the keypad
byte colPins[COLS] = {A3, A2, A1}; //connect to the column pinouts of the keypad

//initialize an instance of class NewKeypad
Keypad customKeypad = Keypad( makeKeymap(hexaKeys), rowPins, colPins, ROWS, COLS); 

LiquidCrystal lcd(7, 6, 5, 4, 3, 2);

const byte midlingAntal = 10;
float laestTemperatur;
float oensketTemperatur = 25.0;
float hysterese = 0.2;
int tempIndtast = 0;
boolean relae = false;
int rawAD[midlingAntal];
byte ptrAD = 0;

const byte tempPin = A0;
const byte varmePin = 13;

int tid = 0;
int state = 0;
int timeout = 0;

void setup() {
  // Sæt input og output til varmestyringen
  pinMode(tempPin, INPUT);
  pinMode(varmePin, OUTPUT);
  // Klargør til Serial
  Serial.begin(9600);
  while (! Serial) { ; }
  Serial.println("Varmestyring");
  Serial.println("------------");
  // Klargør til LCD med 2 linjer 16 karakterer
  lcd.begin(16, 2);
}

void loop() {
  tid++;
  if (tid % 333 == 1) {  // Opdater Display i forhold til statet
    lcd.setCursor(0,0);
    switch (state) {
      case 0:
        lcd.print("Temperatur: ");
        lcd.print(laestTemperatur);
        break;
      case 1:
        lcd.print("Indtast: ");
        lcd.print(tempIndtast);
        lcd.print("       ");
        break;
      case 2:
        lcd.print("Setpunkt: ");
        lcd.print(oensketTemperatur);
        lcd.print("       ");
        break;
      default:
        lcd.print("Fejl i State-m.");
    }
    lcd.setCursor(0,1);
    if (relae) {
      lcd.print("ON ");
    } else {
      lcd.print("OFF");
    }
  }
  if (tid % 200 == 100) {  // Foretag aflæsning og varmeregulering 5 gange i sekundet
    // Læs aktuel AD
    rawAD[ptrAD] = analogRead(tempPin);
    ptrAD = (ptrAD + 1) % midlingAntal;
    // Beregn temperaturen ud fra en midlet AD værdi
    int sum = 0;
    for (int n=0; n < midlingAntal; n++) {
      sum += rawAD[n];
    }
    laestTemperatur = float(sum) / (float(midlingAntal) * 10.0);
    // Reguler ud fra den ønskede temperatur
    if (oensketTemperatur - hysterese < laestTemperatur) {
      relae = true;
    } else if (oensketTemperatur + hysterese > laestTemperatur) {
      relae = false;
    }
    digitalWrite(varmePin, relae);
  }
  if (tid == 1000) {
    tid = 0;
    // Gå tilbage i state 0 ved alt timeout
    if (timeout > 0) {
      timeout--;
    } else {
      state = 0;
    }
    switch (state) {
      case 0:
        Serial.print("Temperatur: ");
        Serial.print(laestTemperatur);
        break;
      case 2:
        Serial.print("Setpunkt: ");
        Serial.print(oensketTemperatur);
        break;
      default:
        Serial.print("Fejl i State-machine.");
    }
    Serial.print("\t");
    Serial.print(relae);
    Serial.print("\t state: ");
    Serial.println(state);
  }
  delay(1);
  // Håndter indtastningen
  char key = customKeypad.getKey();
  if (key) {
    tid = 0;  // Sørg for at visningen kommer med det samme der tastes
    switch(state) {
      case 0:  // Gå ind i indtastning af setpunkt med #
        if (key == '#') {
          state = 1;
          timeout = 20;
        }
        if (key == '*') {  // og i visning af setpunkt med *
          state = 2;
          timeout = 2;
        }
        break;
      case 1:  // opsamling af indtastet tal
        if (key >= '0' && key <= '9') {
          tempIndtast = 10 * tempIndtast + key - '0';
          timeout = 20;
          Serial.print(key);
        } else if (key == '#') {  // Lagring af setpunkt ved #
          oensketTemperatur = float(tempIndtast);
          tempIndtast = 0;
          state = 2;
          timeout = 1;
        } else {  // Ellers er det en fejl
          state = 0;
          tempIndtast = 0;
          timeout = 0;
        }
        break;
      case 2:
        if (key == '*') {
          state = 0;
          timeout = 0;
        }
        break;
      default:
        state = 0;
    }
  }
}
